/*

Programa para integrar numericamente la ecuacion diferencial
del pendulo ideal forzado y con rozamiento:

l * x'' + r * x' + G sin (x) =   f0 * cos ( ome * t);
      
donde l, r, G, f0, ome son parametros que se leen de un archivo
      llamado par_pend.txt 
      t es el tiempo
      x es la funcion del tiempo que se quiere calcular (x(t) es el
      angulo que forma el pendulo con la vertical)
      x'' indica la derivada segunda de x respecto del tiempo y x'
      la derivada primera
      Esta ecuacion necesita dos condiciones iniciales (x(t=0) y x'(t=0)=v_x(t=0))
      a las que llamamos x0 y v0 y que se leen del archivo par_pend.txt
Vean que si eligen r=0 y f0 = 0 se trata de la ecuacion del pendulo ideal
sin forzado y sin rozamiento. 

En este programa se usa un metodo de integracion llamado de Runge-Kutta. 
El metodo de Runge-Kutta puede hacerse mas o menos sofisticado. En este
caso se usa un metodo de cuarto orden. 

La funcion que contiene el integrador Runge-Kutta se llama aca rk4

Para usar este metodo se pasa a dos ecuaciones en derivadas primeras
de la forma:

 x' = v_x;
 v_x' = (-r * v_x - G * sin (x) + f0 * cos ( ome * t))/l;

La parte derecha de estas ecuaciones se escribe dentro de una funcion
que se llama deri:

void deri(n, u, du, t)

El primer argumento (n) es un entero que indica cuantas ecuaciones (en derivadas
primeras) se van a resolver (en este caso dos).  Los otros 3 argumentos son reales 
en doble precision (double). u es un vector que contiene a la posicion (x) y la
velocidad (v_x), du es otro vector que contiene la parte derecha de las 
dos ecuaciones de arriba, t es el tiempo. Para este ejemplo tanto u como du son vectores
de dos componentes. 

La funcion rk4 tiene los siguientes argumentos:
void rk4(h, n, t)
donde h es un vector real de doble precision (en este caso de dos components), n
es un entero (en este caso n=2), t y dt son reales de doble precision e indican
el tiempo y el paso de integracion temporal

*/

#include <stdio.h>
#include <math.h>

/*------DARLE SU VALOR A LAS VARIABLES GLOBALES CON VALORES FIJOS--------------------*/

#define G (9.81)        /*  aceleracion de la gravedad */


/*------DEFINIR EL TIPO DE LAS VARIABLES GLOBALES-------------------------------*/

double l;                /* longitud del pendulo */
double dt;               /* paso de integracion   */
double r;                /* rozamiento */
double t_total;          /* tiempo total de integracion */
double ome;              /* frecuencia del forzado */
double f0;               /* amplitud del forzado */
double x, v_x, x0, v0;
int n_var;               /* numero de variables regidas por ecuaciones diferenciales en 
			    derivadas primeras, en este caso 2 (x y v_x) */ 

void leer_parametros ();
void rk4(double u[2], int n_var, double t);
void deri(int n_var,double u[2],double du[2],double t);

/*----------------FIN DEFINICIONES GLOBALES--------------------------------*/


/* Funcion donde se calcula la derivada para el integrador Runge-Kutta */

void deri(n, u, du, t)
     int n;
     double t;
     double u[2];
     double du[2]; 

{
  x=u[0]; v_x=u[1];

  du[0] = v_x;
  du[1] = (-r * v_x - G * sin (x) + f0 * cos ( ome * t))/l;

}

/* Runge Kutta integrator from numerical recipes plus improvements */


void rk4(h, n, t)
     int n;
     double t;
     double h[2];
{
#define naux 10
  
  int i;
  double k1[naux],k2[naux],k3[naux],k4[naux],h0[naux];
  double dt2, dt6;
  
  dt2=dt/2.;
  dt6=dt/6.;
  
  for (i = 0 ; i<n; i++)
    h0[i] = h[i];
  
  deri(n,h0,k1,t);
  for (i =0 ; i<n ; i++)
    h0[i]=h[i]+dt2*k1[i];
  
  deri(n,h0,k2,t+dt2);
  for (i =0 ; i<n ; i++)
    h0[i]=h[i]+dt2*k2[i];
  
  deri(n,h0,k3,t+dt2);
  for (i =0 ; i<n ; i++)
    h0[i]=h[i]+dt*k3[i];
  
  deri(n,h0,k4,t+dt);
  for (i = 0 ; i<n ; i++)
    {h0[i]=h[i]+dt*k4[i];};
  
  for (i =0; i<n ; i++)
    h[i]=h[i]+dt6*(2.*(k2[i]+k3[i])+k1[i]+k4[i]);
  
  return;
}

/*------------------------------------------------------------------------------
  NAME:              leer_parametros
  DESCRIPTION:       Lee los parametros de un archivo 'par_pend.txt'. El
                     formato es:
  #
  # comentarios
  # 
  l r f0 ome dt t_total
  x0
  v0

  ------------------------------------------------------------------------------*/

void
leer_parametros ()
{
  FILE *fp;
  char buffer[1024];

  fp = fopen ("par_pend.txt", "r");
  if (fp == NULL)
    {
      printf ("No pude abrir el archivo <parametros.txt.\n");
      printf ("Programa abortado.\n");
      exit (-1);
    }

  while (1)
    {
      if (fgets (buffer,1024,fp) == NULL)
	break;
      if (buffer[0] == '#')
	continue;
      else
	break;
    }

  sscanf (buffer, "%lf %lf %lf %lf %lf %lf", &l, &r, &f0, &ome, &dt, 
	  &t_total);
  fscanf (fp, "%lf %lf", &x0, &v0);
  printf ("%lf %lf %lf %lf %lf %lf %lf\n", l, r, f0, ome, dt, 
	  t_total);
  printf ("%lf %lf\n", x0, v0);
}

/*------------------------------------------------------------------------------
  NAME:              main
  DESCRIPTION:       Programa principal. Donde empieza el programa
------------------------------------------------------------------------------*/

void
main ()
{
  int n_pasos;                    /* Numero de pasos de integracion */
  int i;                          /* Variable "temporal" entera */
  double t;                       /* Tiempo real                */ 
  double pi,one,Deltax,fracc;      /* Variables auxiliares       */
  double u[2];
  double du[2]; 
  FILE *datfile,*datfile2;        /* archivos                        */

  n_var = 2;
  /* Abrir los archivos donde vamos a guardar los resultados (pend_rk4 y 
     pend_ef_rk4) */

  datfile = fopen("pendulo_rk4","w");

  datfile2 = fopen("pendulo_ef_rk4","w");

  /* Leer parametros fisicos de un archivo que llamamos par_pend.txt    */
  leer_parametros ();
  
  /* Calculo de pi*/
  
  one=1.0;
  pi=4.*atan(one);


  /* Arrancar la simulacion                                               */
  t = 0.;

  /* Calcular cuantos pasos tenemos que hacer para integrar hasta t_total */
  n_pasos = t_total / dt;

  /* Imponer condiciones iniciales */
  x=x0;
  v_x=v0;
  u[0]=x;
  u[1]=v_x;
  printf ("%lf %lf\n", u[0], u[1]);


  /* Imprimir x, v_x en un archivo llamado pendulo_ef_rk4 */
  fprintf (datfile2,"%f %f \n", x,v_x);

  /* Imprimir t, x, v_x en un archivo llamado pendulo_rk4 */
  fprintf (datfile,"%f %f %f \n", t,x,v_x);
 

  /* Integrar la ecuacion 'n_pasos' veces                                 */
  for ( i = 0; i < n_pasos; i = i + 1)
    {

      /* Llamar al integrador de Runge-Kutta */
      rk4(u,n_var,t);
 
      /* Actualizar el tiempo */
      t = t + dt;

      /* Guardar los resultados en las variables x y v_x*/
      x=u[0];
      v_x=u[1];

      /* Imprimir los resultados en un archivo llamado pendulo_rk4 */
      fprintf (datfile,"%f %f %f \n", t,x,v_x);

      /* Para el diagrama en el espacio de fases
	 convertir el valor del angulo a uno entre -pi y pi*/
      Deltax=x/(2.*pi)+0.5;
      x=modf(Deltax,&fracc);
      if(x > 0)
	x=-pi+2.*pi*x;
      else
	x=pi+2.*pi*x;

      /* Imprimir xx, v_x en un archivo llamado pendulo_ef_rk4 */
      fprintf (datfile2,"%f %f \n", x,v_x);
      
    }
}



